#include "uart.h"
#include "stm32f7xx.h"

uint8_t tx_finished,rx_finished;


void uart_tx_rx_dma_init(void)
{
	//Configure GPIOD PD8 as TX pin
	RCC->AHB1ENR|=RCC_AHB1ENR_GPIODEN;
	GPIOD->MODER|=GPIO_MODER_MODER8_1;
	GPIOD->MODER|=GPIO_MODER_MODER9_1;
	#define AF07 0x07
	GPIOD->AFR[1]|=(AF07<<0);
	GPIOD->AFR[1]|=(AF07<<4);
	//Configure the UART
	RCC->APB1ENR|=RCC_APB1ENR_USART3EN;
	USART3->BRR    = 0x008B; // baud rate 115200 @16MHz
	USART3->CR1|=USART_CR1_TE; //enable TX
	USART3->CR1|=USART_CR1_RE; //enable RX
	USART3->CR3|=USART_CR3_DMAT;//enable DMA_TX
	USART3->CR3|=USART_CR3_DMAR;//enable DMA_RX

	USART3->CR1|=USART_CR1_UE;

	RCC->AHB1ENR|=RCC_AHB1ENR_DMA1EN;
	/*DMA is DMA1
	 * USART_TX DMA
	 * Stream is Stream3
	 * Channel is channel 4
	 * USART_RX
	 * Stream is Stream1
	 * Channel is Channel 4
	 * */

	/*DMA1_Stream3 configuration*/
	DMA1_Stream3->CR &=~(DMA_SxCR_EN);
	while((DMA1_Stream3->CR &(DMA_SxCR_EN)));
	#define ch4 0x04
	DMA1_Stream3->CR|=(ch4<<25)|DMA_SxCR_MINC|DMA_SxCR_DIR_0|DMA_SxCR_TCIE;
	DMA1_Stream3->PAR=(uint32_t)&USART3->TDR;
	NVIC_EnableIRQ(DMA1_Stream3_IRQn);
	/*DMA1_Stream1 configuration*/
	DMA1_Stream1->CR &=~(DMA_SxCR_EN);
	while((DMA1_Stream1->CR &(DMA_SxCR_EN)));
	#define ch4 0x04
	DMA1_Stream1->CR|=(ch4<<25)|DMA_SxCR_MINC|DMA_SxCR_TCIE|DMA_SxCR_CIRC;
	DMA1_Stream1->PAR=(uint32_t)&USART3->RDR;
	NVIC_EnableIRQ(DMA1_Stream1_IRQn);

}



void launch_rx_dma(uint8_t * buffer, uint16_t length)
	{
		DMA1_Stream1->M0AR= buffer;
		DMA1_Stream1->NDTR=length;
		DMA1_Stream1->CR|=DMA_SxCR_EN;

	}



uint8_t isBufferFull()
	{
		if(rx_finished==0)	{return buffer_empty;}
		else {rx_finished=0; return buffer_full;}
	}


void usart_dma_write(char *data, uint16_t length)
	{
		DMA1_Stream3->M0AR= data;
		DMA1_Stream3->NDTR=length;
		DMA1_Stream3->CR|=DMA_SxCR_EN;
		while(tx_finished==0);
		tx_finished=0;
	}


void DMA1_Stream3_IRQHandler(void)
	{
		if(DMA1->LISR & DMA_LISR_TCIF3)
			{
				tx_finished=1;
				DMA1->LIFCR=DMA_LIFCR_CTCIF3;
			}

	}



void DMA1_Stream1_IRQHandler(void)
	{
		if(DMA1->LISR & DMA_LISR_TCIF1)
			{
				rx_finished=1;
				DMA1->LIFCR=DMA_LIFCR_CTCIF1;
			}

	}
